from .load import load

# This routine was originally in st5/__init__.py, until being moved to its own file.
# Please refer to __init__.py if you need to see the revision history before it was moved.

def mag_postprocessing(variables):
    """
    Placeholder for MAG post-processing
    """
    return variables
