from .load import load
from .lena import lena
from .mena import mena
from .hena import hena
from .rpi import rpi
from .euv import euv
from .fuv import fuv
from .orbit import orbit
from .datasets import datasets
