import pyspedas.projects.maven.spdf as spdf_load
from .maven_load import load_data
maven_load = load_data
from .euv import euv
from .iuv import iuv
from .kp import kp
from .lpw import lpw
from .mag import mag
from .ngi import ngi
from .rse import rse
from .sep import sep
from .sta import sta
from .swea import swea
from .swia import swia
